# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: user_config.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='user_config.proto',
  package='googleapis.artman',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x11user_config.proto\x12\x11googleapis.artman\"l\n\nUserConfig\x12-\n\x05local\x18\x01 \x01(\x0b\x32\x1e.googleapis.artman.LocalConfig\x12/\n\x06github\x18\x02 \x01(\x0b\x32\x1f.googleapis.artman.GitHubConfig\"\x1e\n\x0bLocalConfig\x12\x0f\n\x07toolkit\x18\x01 \x01(\t\"/\n\x0cGitHubConfig\x12\x10\n\x08username\x18\x01 \x01(\t\x12\r\n\x05token\x18\x02 \x01(\tb\x06proto3')
)




_USERCONFIG = _descriptor.Descriptor(
  name='UserConfig',
  full_name='googleapis.artman.UserConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='local', full_name='googleapis.artman.UserConfig.local', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='github', full_name='googleapis.artman.UserConfig.github', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=40,
  serialized_end=148,
)


_LOCALCONFIG = _descriptor.Descriptor(
  name='LocalConfig',
  full_name='googleapis.artman.LocalConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='toolkit', full_name='googleapis.artman.LocalConfig.toolkit', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=150,
  serialized_end=180,
)


_GITHUBCONFIG = _descriptor.Descriptor(
  name='GitHubConfig',
  full_name='googleapis.artman.GitHubConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='username', full_name='googleapis.artman.GitHubConfig.username', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='token', full_name='googleapis.artman.GitHubConfig.token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=182,
  serialized_end=229,
)

_USERCONFIG.fields_by_name['local'].message_type = _LOCALCONFIG
_USERCONFIG.fields_by_name['github'].message_type = _GITHUBCONFIG
DESCRIPTOR.message_types_by_name['UserConfig'] = _USERCONFIG
DESCRIPTOR.message_types_by_name['LocalConfig'] = _LOCALCONFIG
DESCRIPTOR.message_types_by_name['GitHubConfig'] = _GITHUBCONFIG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

UserConfig = _reflection.GeneratedProtocolMessageType('UserConfig', (_message.Message,), dict(
  DESCRIPTOR = _USERCONFIG,
  __module__ = 'user_config_pb2'
  # @@protoc_insertion_point(class_scope:googleapis.artman.UserConfig)
  ))
_sym_db.RegisterMessage(UserConfig)

LocalConfig = _reflection.GeneratedProtocolMessageType('LocalConfig', (_message.Message,), dict(
  DESCRIPTOR = _LOCALCONFIG,
  __module__ = 'user_config_pb2'
  # @@protoc_insertion_point(class_scope:googleapis.artman.LocalConfig)
  ))
_sym_db.RegisterMessage(LocalConfig)

GitHubConfig = _reflection.GeneratedProtocolMessageType('GitHubConfig', (_message.Message,), dict(
  DESCRIPTOR = _GITHUBCONFIG,
  __module__ = 'user_config_pb2'
  # @@protoc_insertion_point(class_scope:googleapis.artman.GitHubConfig)
  ))
_sym_db.RegisterMessage(GitHubConfig)


# @@protoc_insertion_point(module_scope)
